from openfisca_tunisia.variables.base import *  # noqa analysis:ignore


class nb_enf(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Nombre d'enfants"
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        '''
        TODO: fixme
        '''
        age = foyer_fiscal.members('age', period = period.first_month)
        famille = parameters(period.start).impot_revenu.deductions.fam
        # res =+ (
        #    (ag < 20) +
        #    (ag < 25)*not_(boursier)*()
        #    )
        condition = (age >= 0) * (age <= famille.age)
        return foyer_fiscal.sum(condition, role = FoyerFiscal.PERSONNE_A_CHARGE)


class nb_enf_sup(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Nombre d'enfants étudiants du supérieur non boursiers"
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        '''
        TODO: Nombre d'enfants étudiants du supérieur non boursiers
        '''
        # age = foyer_fiscal.members('age', period = period)
        etudiant = foyer_fiscal.members('etudiant', period = period)
        boursier = foyer_fiscal.members('boursier', period = period)

        return foyer_fiscal.sum(etudiant * not_(boursier))


class nb_infirme(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Nombre d'enfants infirmes"
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        '''
        TODO: Nombre d'enfants infirmes
        '''
        age = foyer_fiscal.members('age', period = period)
        handicap = foyer_fiscal.members('handicap', period = period)

        return 0 * age * handicap


class nb_parents(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Nombre de parents'
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        '''
        TODO: Nombre de parents
        '''
        return (
            (foyer_fiscal.declarant_principal('age', period) > 20)
            + (foyer_fiscal.declarant_principal('age', period) > 20)
            )


class chef_de_famille(Variable):
    value_type = bool
    entity = FoyerFiscal
    label = 'Indicatrice de chef de famille'
    definition_period = YEAR
    set_input = set_input_dispatch_by_period

    # Du point de vue fiscal, est considéré chef de famille :
    # - L’époux ;
    # - Le divorcé ou la divorcée qui a la garde des enfants (divorce & enfnats);
    # - Le veuf ou la veuve même sans enfants à charge;
    # - L’adoptant ou l’adoptante (adoptant). TODO
    # Cependant, l’épouse a la qualité de chef de famille dans les deux cas suivants :
    # - Lorsqu’elle justifie que le mari ne dispose d’aucune source de revenu durant l’année de réalisation du
    #   revenu. Tel est le cas d’une femme qui dispose d’un revenu et dont le mari, poursuivant des études, ne dispose
    #   d’aucun revenu propre. (marie & sexe & revenu_epoux == 0 & revenu_individu > 0) TODO
    # - Lorsque remariée, elle a la garde d’enfants issus d’un précédent mariage. TODO
    # Compte tenu de ce qui précède, n’est pas considéré comme chef de famille et ne bénéficie d’aucune déduction :
    # - Le célibataire ou la célibataire ;
    # - Le divorcé ou la divorcée qui n’a pas la garde des enfants ;
    # - La femme durant le mariage (sauf si elle dispose d’un revenu alors que son mari ne dispose d’aucun revenu) ;
    # - L’époux qui ne dispose pas d’une source de revenu. Dans ce cas, l’épouse acquiert la qualité de chef de famille
    #   au cas où elle réalise des revenus.

    def formula(foyer_fiscal, period):
        male = foyer_fiscal.declarant_principal('male', period = period)
        marie = foyer_fiscal.declarant_principal('marie', period = period)
        divorce = foyer_fiscal.declarant_principal('divorce', period = period)
        veuf = foyer_fiscal.declarant_principal('veuf', period = period)
        nb_enf = foyer_fiscal('nb_enf', period = period)
        chef_de_famille = (
            veuf | (marie & male) | (divorce & (nb_enf > 0))  # | (marie & (not male)) |
            )
        return chef_de_famille

# Revenus catégoriels (voir répertoire idoine)
# 1. Bénéfices industriels et commerciaux  bic.py
# 2. Bénéfices des professions non commerciales  bnc.py
# 3. Bénéfices de l'exploitation agricole et de pêche beap.py
# 4. Revenus fonciers foncier.py
# 5. Traitements, salaires, indemnités, pensions et rentes viagères  tspr.py
# 6. Revenus de valeurs mobilières et de capitaux mobiliers  rvcm.py
# 7. revenus de source étrangère

# Déroulé du calcul de l'irpp


class rng(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Revenu net global'
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        return (
            foyer_fiscal('bic', period = period)
            + foyer_fiscal('bnc', period = period)
            + foyer_fiscal('beap', period = period)
            + foyer_fiscal('revenus_fonciers', period = period)
            + foyer_fiscal('tspr', period = period)
            + foyer_fiscal('rvcm', period = period)
            + foyer_fiscal('revenus_source_etrangere', period = period)
            )


#############################
#    Déductions
#############################

# # 1/ Au titre des revenus et bénéfices provenant de l’activité

# # 2/ Autres déductions


class deduction_interets(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Déductions intérêts issus de comptes spéciaux ou d'obligations"
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        compte_special_epargne_banque = foyer_fiscal('compte_special_epargne_banque')
        compte_special_epargne_cent = foyer_fiscal('compte_special_epargne_cent')
        emprunt_obligataire = foyer_fiscal('emprunt_obligataire')
        deductions = parameters(period).impot_revenu.deductions
        return max_(
            max_(
                (
                    max_(compte_special_epargne_banque, deductions.banq.plaf)
                    + max_(compte_special_epargne_cent, deductions.cent.plaf)
                    ),
                deductions.banq.plaf,
                )
            + max_(emprunt_obligataire, deductions.oblig.plaf),
            deductions.oblig.plaf,
            )


class deduction_famille(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Déductions pour situation et charges de famille'
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        # rng = foyer_fiscal('rng', period = period)
        chef_de_famille = foyer_fiscal('chef_de_famille', period = period)
        nb_enf = foyer_fiscal('nb_enf', period = period)
        # nb_parents = foyer_fiscal('nb_parents', period = period)
        fam = parameters(period.start).impot_revenu.deductions.fam
        #  chef de famille
        chef_de_famille = fam.chef_de_famille * chef_de_famille

        enf = (nb_enf >= 1) * fam.enf1 + (nb_enf >= 2) * fam.enf2 + (nb_enf >= 3) * fam.enf3 + (nb_enf >= 4) * fam.enf4

        #    sup = P.enf_sup * nb_enf_sup
        #    infirme = P.infirme * nb_infirme
        #    parent = min_(P.parent_taux * rng, P.parent_max)

        #    return chef_de_famille + enf + sup + infirme + parent
        res = chef_de_famille + enf
        return res


class deduction_rente(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Arrérages et rentes payées à titre obligatoire et gratuit'
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        rente = foyer_fiscal('rente', period = period)

        return rente  # TODO:


class deduction_assurance_vie(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Primes afférentes aux contrats d'assurance-vie collectifs ou individuels"
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        primes_assurance_vie = foyer_fiscal.members('prime_assurance_vie', period = period)
        somme_primes_assurance_vie = foyer_fiscal.sum(primes_assurance_vie)
        marie = foyer_fiscal.declarant_principal('statut_marital', period = period)
        nb_enf = foyer_fiscal('nb_enf', period = period)
        assurance_vie = parameters(period.start).impot_revenu.deductions.assurance_vie
        deduction = min_(
            somme_primes_assurance_vie,
            assurance_vie.plaf + marie * assurance_vie.conj_plaf + nb_enf * assurance_vie.enf_plaf)
        return deduction


#     - Les remboursements des prêts universitaires en principal et en intérêts
#    - Revenus et bénéfices réinvestis dans les conditions et limites
#    prévues par la législation en vigueur dont notamment :
#     les revenus provenant de l'exportation, totalement pendant 10
#    ans à partir de la première opération d'exportation;
#     les revenus provenant de l’hébergement et de la restauration
#    des étudiants pendant dix ans ;
#     les revenus provenant du courtage international dans la limite
#    de 50% pendant 10 ans à partir de la première opération de courtage
#    international;
#     les montants déposés dans les comptes-épargne pour
#    l’investissement et dans les comptes-épargne en actions dans la
#    limite de 20.000D par an et sous réserve du minimum d’IR ;
#     la plus-value provenant des opérations de transmission des
#    entreprises en difficultés économiques ou des entreprises sous
#    forme de participations ou d’actifs pour départ du propriétaire à la
#    retraite ou pour incapacité de poursuivre la gestion et ce, sous
#    certaines conditions,
#     la plus-value provenant de l’apport d’une entreprise individuelle
#    au capital d’une société ;
#     les revenus réinvestis dans la souscription au capital des
#    entreprises dans les conditions prévues par la législation relative aux
#    incitations fiscales;
#     les revenus réinvestis dans la réalisation de projets
#    d’hébergement et de restauration universitaires sous réserve du
#    minimum d’IR.
#     Les revenus provenant de la location des terres agricoles
#    réservées aux grandes cultures objet de contrat de location conclus
#    pour une période minimale de trois ans.


class deduction_smig(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Déduction supplémentaire pour les salariés payés au SMIG et SMAG'
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        chef = foyer_fiscal('chef_de_famille', period = period)

        return 0 * chef  # TODO: voir avec tspr


class deductions(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Revenu net imposable'
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        return (
            foyer_fiscal('deduction_famille', period = period)
            + foyer_fiscal.declarant_principal('rente', period = period)
            + foyer_fiscal('deduction_assurance_vie', period = period)
            + foyer_fiscal.sum(foyer_fiscal.members('plus_value_cession_actifs_cotes_bourse', period = period))
            + foyer_fiscal.sum(foyer_fiscal.members('interets_acquisition_logement', period = period))
            )


class revenu_net_imposable(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Revenu net imposable'
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        '''
        Revenu net imposable ie soumis à au barême de l'impôt après déduction des dépenses
        et charges professionnelles et des revenus non soumis à l'impôt
        '''
        return max_(
            foyer_fiscal('rng', period) - foyer_fiscal('deductions', period),
            0
            )


class impot_revenu_brut(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Impôt brut avant non-imposabilité'
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        revenu_net_imposable = foyer_fiscal('revenu_net_imposable', period = period)
        bareme = parameters(period.start).impot_revenu.bareme
        impot_revenu_brut = bareme.calc(revenu_net_imposable)
        return impot_revenu_brut


class exoneration(Variable):
    value_type = bool
    entity = FoyerFiscal
    label = "Exoneration de l'impôt sur le revenu des personnes physiques"
    definition_period = YEAR
    end = '2016-12-31'

    def formula_2014(foyer_fiscal, period, parameters):
        # Les éligibles ne doivent percevoir que des salaires et des pensions
        rng = foyer_fiscal('rng', period = period)
        tspr = foyer_fiscal('tspr', period = period)
        eligble = (rng == tspr)
        # Condition de revenu
        deduction_famille = foyer_fiscal('deduction_famille', period = period)
        condition_de_revenu = (
            rng - deduction_famille
            ) <= parameters(period.start).impot_revenu.exoneration.seuil
        return eligble * condition_de_revenu


class irpp(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Impôt sur le revenu des personnes physiques'
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        impot_revenu_brut = foyer_fiscal('impot_revenu_brut', period = period)
        exoneration = foyer_fiscal('exoneration', period = period)
        return impot_revenu_brut * not_(exoneration)


class revenu_mensuel_assimile_salaire_apres_abattement(Variable):
    value_type = float
    entity = Individu
    label = 'Impôt sur le revenu des personnes physiques prélevé à la source pour les salariés'
    definition_period = MONTH

    def formula_2011(foyer_fiscal, period):
        revenu_assimile_salaire = individu('salaire_imposable', period = period)
        smig_40h_mensuel = parameters(period.start).marche_travail.smig_40h_mensuel
        smig = revenu_assimile_salaire <= smig_40h_mensuel
        tspr = parameters(period.start).impot_revenu.tspr

        abattement_frais_professionnels = min_(revenu_assimile_salaire * tspr.abat_sal, tspr.max_abat_sal)
        revenu_assimile_salaire_apres_abattement = max_(
            revenu_assimile_salaire
            - abattement_frais_professionnels
            - min_(
                smig * tspr.abattement_pour_salaire_minimum,
                (revenu_assimile_salaire <= tspr.smig_ext) * tspr.abattement_pour_salaire_minimum
                ),
            0)
        return revenu_assimile_salaire_apres_abattement

    def formula(foyer_fiscal, period):
        revenu_assimile_salaire = individu('salaire_imposable', period = period)
        smig_40h_mensuel = parameters(period.start).marche_travail.smig_40h_mensuel
        smig = revenu_assimile_salaire <= smig_40h_mensuel
        tspr = parameters(period.start).impot_revenu.tspr
        abattement_frais_professionnels = min_(revenu_assimile_salaire * tspr.abat_sal, tspr.max_abat_sal)
        revenu_assimile_salaire_apres_abattement = max_(
            revenu_assimile_salaire
            - abattement_frais_professionnels
            - smig * tspr.abattement_pour_salaire_minimum,
            0
            )
        return revenu_assimile_salaire_apres_abattement


class irpp_salarie_preleve_a_la_source(Variable):
    value_type = float
    entity = Individu
    label = 'Impôt sur le revenu des personnes physiques prélevé à la source pour les salariés'
    definition_period = MONTH

    def formula(individu, period, parameters):
        salaire_imposable = individu('salaire_imposable', period = period)
        deduction_famille_annuelle = individu.foyer_fiscal('deduction_famille', period = period.this_year)

        return calcule_impot_revenu_brut(
            salaire_imposable, deduction_famille_annuelle, period, parameters,
            )


class irpp_net_a_payer(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'IRPP net des prélèvements à la source'
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        return (
            foyer_fiscal('irpp', period)
            - foyer_fiscal.sum(foyer_fiscal.members('irpp_salarie_preleve_a_la_source', period, options = [ADD]))
            )


# Utils

def calcule_base_imposable(salaire_mensuel, deduction_famille_annuelle, period, parameters):
    revenu_assimile_salaire = salaire_mensuel
    smig_40h_mensuel = parameters(period.start).marche_travail.smig_40h_mensuel
    smig = revenu_assimile_salaire <= smig_40h_mensuel
    tspr = parameters(period.start).impot_revenu.tspr
    abattement_frais_professionnels = min_(
        tspr.abat_sal * revenu_assimile_salaire,
        tspr.max_abat_sal / 12
        )

    if period.start.year >= 2011:
        revenu_assimile_salaire_apres_abattement = max_(
            revenu_assimile_salaire
            - abattement_frais_professionnels
            - max_(
                smig * tspr.abattement_pour_salaire_minimum,
                (revenu_assimile_salaire <= tspr.smig_ext) * tspr.abattement_pour_salaire_minimum),
            0
            )
    else:
        revenu_assimile_salaire_apres_abattement = max_(
            revenu_assimile_salaire
            - abattement_frais_professionnels
            - smig * tspr.abattement_pour_salaire_minimum,
            0)

    non_exonere = revenu_assimile_salaire_apres_abattement >= 0
    if 2014 <= period.start.year <= 2016:
        non_exonere = (
            (12 * revenu_assimile_salaire_apres_abattement - deduction_famille_annuelle)
            ) > parameters(period.start).impot_revenu.exoneration.seuil

    return non_exonere, revenu_assimile_salaire_apres_abattement


def calcule_impot_revenu_brut(salaire_mensuel, deduction_famille_annuelle, period, parameters):
    non_exonere, revenu_assimile_salaire_apres_abattement = calcule_base_imposable(
        salaire_mensuel, deduction_famille_annuelle, period, parameters)
    bareme = parameters(period.start).impot_revenu.bareme
    return (
        non_exonere * bareme.calc(
            12 * revenu_assimile_salaire_apres_abattement - deduction_famille_annuelle
            )
        ) / 12



'''7. revenus de source étrangère'''


from openfisca_tunisia.variables.base import *  # noqa analysis:ignore


class revenus_source_etrangere(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Autres revenus (revenus de source étrangère n’ayant pas subi l’impôt dans le pays d'origine)"
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        salaire_etranger = foyer_fiscal.declarant_principal('salaire_etranger', period = period)
        pension_etranger_transferee = foyer_fiscal.declarant_principal(
            'pension_etranger_transferee', period = period)
        pension_etranger_non_transferee = foyer_fiscal.declarant_principal(
            'pension_etranger_non_transferee', period = period)
        autres_revenus_etranger = foyer_fiscal.declarant_principal(
            'autres_revenus_etranger', period = period)
        tspr = parameters(period.start).impot_revenu.tspr
        abattement_frais_professionnels = min_(salaire_etranger * tspr.abat_sal, tspr.max_abat_sal)

        return (
            salaire_etranger - abattement_frais_professionnels
            + pension_etranger_non_transferee * (1 - tspr.abat_pen)
            + pension_etranger_transferee * (1 - tspr.abat_pen_etr)
            + autres_revenus_etranger
            )




from openfisca_tunisia.variables.base import *  # noqa analysis:ignore


class TypesRegimeSecuriteSocialeCotisant(Enum):
    __order__ = 'rsna rsa rsaa rtns rtte re rtfr raci salarie_cnrps pensionne_cnrps neant'
    # Needed to preserve the enum order in Python 2

    rsna = 'Régime des salariés non agricoles'
    rsa = 'Régime des salariés agricoles'
    rsaa = 'Régime des salariés agricoles amélioré'
    rtns = 'Régime des travailleurs non salariés (secteurs agricole et non agricole)'
    rtte = "Régime des travailleurs tunisiens à l'étranger"
    re = "Régime des étudiants, diplômés de l'enseignement supérieur et stagiaires"
    rtfr = 'Régime des travailleurs à faibles revenus (gens de maisons, travailleurs de chantiers, et artisans travaillant à la pièce)'
    raci = 'Régime des artistes, créateurs et intellectuels'
    salarie_cnrps = 'Régime des salariés affilés à la Caisse nationale de retraite et de rrévoyance sociale (CNRPS)'
    pensionne_cnrps = 'Régime des pensionnés affilés à la Caisse nationale de retraite et de rrévoyance sociale (CNRPS)'
    neant = 'Néant'
    # references :
    # http://www.social.gov.tn/index.php?id=49&L=0
    # http://www.paie-tunisie.com/412/fr/83/reglementations/regimes-de-securite-sociale.aspx


class TypesRegimeSecuriteSocialeRetraite(Enum):
    __order__ = 'rsna rtte raci rtns_na rtns_a rtc rsa rsaa cnrps neant'
    # Needed to preserve the enum order in Python 2

    rsna = 'Régime des salariés non agricoles'
    rtte = "Régime des travailleurs tunisiens à l'étranger"
    raci = 'Régime des artistes, créateurs etiIntellectuels'
    rtns_na = 'Régime des travailleurs non salariés des secteurs non agricoles'
    rtns_a = 'Régime des travailleurs non salariés des secteurs agricoles'
    rtc = 'Régime des travailleurs de chantiers'
    rsa = 'Régime des salariés agricoles'
    rsaa = 'Régime des salariés agricoles amélioré'
    cnrps = 'Régime des pensionnés affilés à la Caisse Nationale de Retraite et de Prévoyance Sociale'
    neant = 'Néant'
    # references :
    # http://www.social.gov.tn/index.php?id=49&L=0
    # http://www.paie-tunisie.com/412/fr/83/reglementations/regimes-de-securite-sociale.aspx


def compute_cotisation(individu, period, cotisation_type = None, bareme_name = None, parameters = None):
    assert cotisation_type in ['employeur', 'salarie']

    assiette_cotisations_sociales = individu('assiette_cotisations_sociales', period)
    regime_securite_sociale_cotisant = individu('regime_securite_sociale_cotisant', period)
    baremes_by_regime = parameters(period.start).prelevements_sociaux.cotisations_sociales
    cotisation = individu.empty_array()
    types_regime_securite_sociale_cotisant = regime_securite_sociale_cotisant.possible_values

    for regime in types_regime_securite_sociale_cotisant:
        if regime.name == 'neant':
            continue
        if 'cotisations_{}'.format(cotisation_type) not in baremes_by_regime[regime.name]._children:
            continue
        baremes_by_name = getattr(
            baremes_by_regime[regime.name],
            'cotisations_{}'.format(cotisation_type),
            )

        if bareme_name in ['maladie', 'maternite', 'deces']:
            if 'assurances_sociales' in baremes_by_name._children:
                baremes_assurances_sociales = getattr(baremes_by_name, 'assurances_sociales')
                bareme = getattr(baremes_assurances_sociales, bareme_name)

            else:
                if bareme_name not in baremes_by_name._children:
                    continue
                bareme = getattr(baremes_by_name, bareme_name)

        else:
            if bareme_name not in baremes_by_name._children:
                continue
            bareme = getattr(baremes_by_name, bareme_name)

        if bareme is not None:
            cotisation += bareme.calc(
                assiette_cotisations_sociales * (regime_securite_sociale_cotisant == regime),
                )

    return cotisation


class assiette_cotisations_sociales(Variable):
    value_type = float
    entity = Individu
    label = 'Assiette des cotisations sociales'
    definition_period = MONTH

    def formula(individu, period):
        return (
            individu('salaire_de_base', period)
            + individu('primes', period)
            )


class regime_securite_sociale_cotisant(Variable):
    value_type = Enum
    possible_values = TypesRegimeSecuriteSocialeCotisant
    default_value = TypesRegimeSecuriteSocialeCotisant.neant
    entity = Individu
    label = 'Régime de sécurité sociale du salarié'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class regime_securite_sociale_retraite(Variable):
    value_type = Enum
    possible_values = TypesRegimeSecuriteSocialeRetraite
    default_value = TypesRegimeSecuriteSocialeRetraite.neant
    entity = Individu
    label = 'Régime de sécurité sociale du salarié'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class cotisations_sociales(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisations sociales'
    definition_period = MONTH

    def formula(individu, period):
        return (
            individu('cotisations_employeur', period)
            + individu('cotisations_salarie', period)
            )


class cotisations_employeur(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation sociales employeur'
    definition_period = MONTH

    def formula(individu, period):
        return (
            individu('accident_du_travail_employeur', period)
            + individu('deces_employeur', period)
            + individu('fonds_special_etat', period)
            + individu('famille_employeur', period)
            + individu('maladie_employeur', period)
            + individu('maternite_employeur', period)
            + individu('protection_sociale_travailleurs_employeur', period)
            + individu('retraite_employeur', period)
            )


class cotisations_salarie(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation sociales salarié'
    definition_period = MONTH

    def formula(individu, period):
        return (
            individu('accident_du_travail_salarie', period)
            + individu('deces_salarie', period)
            + individu('famille_salarie', period)
            + individu('maladie_salarie', period)
            + individu('maternite_salarie', period)
            + individu('protection_sociale_travailleurs_salarie', period)
            + individu('retraite_salarie', period)
            + individu('ugtt', period)
            )


class accident_du_travail_employeur(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation accidents du travail et maladies professionnelles (employeur)'
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type = 'employeur',
            bareme_name = 'accident_du_travail',
            parameters = parameters
            )


class accident_du_travail_salarie(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation accidents du travail et maladies professionnelles (salarié)'
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type = 'salarie',
            bareme_name = 'accident_du_travail',
            parameters = parameters
            )


class deces_employeur(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation assurances sociales: décès (employeur)'
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type = 'employeur',
            bareme_name = 'deces',
            parameters = parameters
            )


class deces_salarie(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation assurances sociales: décès (salarié)'
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type = 'salarie',
            bareme_name = 'deces',
            parameters = parameters
            )


class famille_employeur(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation sociale allocations familiales (employeur)'
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type = 'employeur',
            bareme_name = 'famille',
            parameters = parameters
            )


class famille_salarie(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation sociale allocations familiales (salarié)'
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type = 'salarie',
            bareme_name = 'famille',
            parameters = parameters
            )


class fonds_special_etat(Variable):
    value_type = float
    entity = Individu
    label = "Fonds spécial de l'Etat"
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type = 'employeur',
            bareme_name = 'fonds_special_etat',
            parameters = parameters
            )


class maladie_employeur(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation assurances sociales: maladie (employeur)'
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type = 'employeur',
            bareme_name = 'maladie',
            parameters = parameters
            )


class maladie_salarie(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation assurances sociales: maladie (salarié)'
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type = 'salarie',
            bareme_name = 'maladie',
            parameters = parameters
            )


class maternite_employeur(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation assurances sociales : maternité (employeur)'
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type = 'employeur',
            bareme_name = 'maternite',
            parameters = parameters
            )


class maternite_salarie(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation assurances sociales : maternité (salarié)'
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type = 'salarie',
            bareme_name = 'maternite',
            parameters = parameters
            )


class protection_sociale_travailleurs_employeur(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation protection sociale des travailleurs (employeur)'
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type = 'employeur',
            bareme_name = 'protection_sociale_travailleurs',
            parameters = parameters
            )


class protection_sociale_travailleurs_salarie(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation protection sociale des travailleurs (salarié)'
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type = 'salarie',
            bareme_name = 'protection_sociale_travailleurs',
            parameters = parameters
            )


class retraite_employeur(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation pensions de retraite (employeur)'
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type = 'employeur',
            bareme_name = 'retraite',
            parameters = parameters
            )


class retraite_salarie(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation pensions de retraite (salarié)'
    definition_period = MONTH

    def formula(individu, period, parameters):
        return compute_cotisation(
            individu,
            period,
            cotisation_type = 'salarie',
            bareme_name = 'retraite',
            parameters = parameters
            )


class salaire_imposable(Variable):
    value_type = float
    entity = Individu
    label = 'Salaire imposable'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period):
        return (
            individu('assiette_cotisations_sociales', period)
            - individu('cotisations_salarie', period)
            )


class salaire_net_a_payer(Variable):
    value_type = float
    entity = Individu
    label = 'Salaire net à payer (fiche de paie)'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2018_01_01(individu, period):
        return (
            individu('salaire_imposable', period)
            - individu('irpp_salarie_preleve_a_la_source', period)
            - individu('contribution_sociale_solidarite_prelevee_a_la_source', period)
            )

    def formula(individu, period):
        return (
            individu('salaire_imposable', period)
            - individu('irpp_salarie_preleve_a_la_source', period)
            )


class salaire_super_brut(Variable):
    value_type = float
    entity = Individu
    label = 'Salaires super bruts'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period):
        return (
            individu('salaire_de_base', period = period)
            + individu('primes', period = period)
            + individu('cotisations_employeur', period = period)  # Cotisations employeur sont négatives
            )


class ugtt(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisation syndicale UGTT'
    definition_period = MONTH

    def formula(individu, period):
        # TODO put this value (3) in parameters
        return 3 * (individu('regime_securite_sociale_cotisant', period) == TypesRegimeSecuriteSocialeCotisant.salarie_cnrps)



from numpy import datetime64, timedelta64


from openfisca_tunisia.variables.base import *


class age(Variable):
    unit = 'years'
    value_type = int
    default_value = AGE_INT_MINIMUM
    entity = Individu
    label = 'Âge (en années) au premier jour du mois'
    definition_period = MONTH
    is_period_size_independent = True
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        has_birth = individu.get_holder('date_naissance').get_known_periods()
        if not has_birth:
            has_age_en_mois = bool(individu.get_holder('age_en_mois').get_known_periods())
            if has_age_en_mois:
                return individu('age_en_mois', period) // 12

            # If age is known at the same day of another year, compute the new age from it.
            holder = individu.get_holder('age')
            start = period.start
            known_periods = holder.get_known_periods()
            if known_periods:
                for last_period in sorted(known_periods, reverse = True):
                    last_start = last_period.start
                    if last_start.day == start.day:
                        last_array = holder.get_array(last_period)
                        return (
                            last_array
                            + int(
                                start.year
                                - last_start.year
                                + (start.month - last_start.month) / 12
                                )
                            )

        date_naissance = individu('date_naissance', period)
        epsilon = timedelta64(1)
        return (datetime64(period.start) - date_naissance + epsilon).astype('timedelta64[Y]')


class age_en_mois(Variable):
    value_type = int
    default_value = AGE_INT_MINIMUM
    unit = 'months'
    entity = Individu
    label = 'Âge (en mois)'
    is_period_size_independent = True
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        # If age_en_mois is known at the same day of another month, compute the new age_en_mois from it.
        holder = individu.get_holder('age_en_mois')
        start = period.start
        known_periods = holder.get_known_periods()

        for last_period in sorted(known_periods, reverse = True):
            last_start = last_period.start
            if last_start.day == start.day:
                last_array = holder.get_array(last_period)
                return last_array + ((start.year - last_start.year) * 12 + (start.month - last_start.month))

        has_birth = individu.get_holder('date_naissance').get_known_periods()
        if not has_birth:
            has_age = bool(individu.get_holder('age').get_known_periods())
            if has_age:
                return individu('age', period) * 12
        date_naissance = individu('date_naissance', period)
        epsilon = timedelta64(1)
        return (datetime64(period.start) - date_naissance + epsilon).astype('timedelta64[M]')


class date_naissance(Variable):
    value_type = date
    default_value = date(1970, 1, 1)
    entity = Individu
    label = 'Date de naissance'
    definition_period = ETERNITY


class male(Variable):
    value_type = bool
    entity = Individu
    label = 'Mâle'
    definition_period = ETERNITY


class marie(Variable):
    value_type = bool
    entity = Individu
    label = 'Marié(e)'
    definition_period = YEAR

    def formula(individu, period):
        statut_marital = individu('statut_marital', period = period)
        return (statut_marital == 1)


class celibataire(Variable):
    value_type = bool
    entity = Individu
    label = 'Célibataire'
    definition_period = YEAR

    def formula(individu, period):
        statut_marital = individu('statut_marital', period = period)
        return (statut_marital == 2)


class divorce(Variable):
    value_type = bool
    entity = Individu
    label = 'Divorcé(e)'
    definition_period = YEAR

    def formula(individu, period):
        statut_marital = individu('statut_marital', period = period)
        return (statut_marital == 3)


class veuf(Variable):
    value_type = bool
    entity = Individu
    label = 'Veuf(ve)'
    definition_period = YEAR

    def formula(individu, period):
        statut_marital = individu('statut_marital', period = period)
        return statut_marital == 4


class statut_marital(Variable):
    value_type = int
    default_value = 2
    entity = Individu
    label = 'Statut marital'
    definition_period = YEAR



'''3. Bénéfices de l'exploitation agricole et de pêche.'''


from openfisca_tunisia.variables.base import *  # noqa analysis:ignore


class beap(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Bénéfices de l'exploitation agricole et de pêche (BEAP)"
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        beap_reel_res_fiscal = foyer_fiscal.sum(foyer_fiscal.members('beap_reel_res_fiscal', period = period))
        beap_reliq_benef_fiscal = foyer_fiscal.sum(foyer_fiscal.members('beap_reliq_benef_fiscal', period = period))
        beap_monogr = foyer_fiscal.sum(foyer_fiscal.members('beap_monogr', period = period))
        beap_part_benef_sp = foyer_fiscal.sum(foyer_fiscal.members('beap_part_benef_sp', period = period))

        return beap_reel_res_fiscal + beap_reliq_benef_fiscal + beap_monogr + beap_part_benef_sp



import argparse
import requests
import logging


logging.basicConfig(level=logging.INFO)


def get_info(package_name: str = '') -> dict:
    '''
    Get minimal informations needed by .conda/meta.yaml from PyPi JSON API.
    ::package_name:: Name of package to get infos from.
    ::return:: A dict with last_version, url and sha256
    '''
    if package_name == '':
        raise ValueError('Package name not provided.')
    resp = requests.get(f'https://pypi.org/pypi/{package_name}/json').json()
    version = resp['info']['version']
    for v in resp['releases'][version]:
        if v['packagetype'] == 'sdist':  # for .tag.gz
            return {
                'last_version': version,
                'url': v['url'],
                'sha256': v['digests']['sha256']
                }


def replace_in_file(filepath: str, info: dict):
    '''
    ::filepath:: Path to meta.yaml, with filename
    ::info:: Dict with information to populate
    '''
    with open(filepath, 'rt') as fin:
        meta = fin.read()
    # Replace with info from PyPi
    meta = meta.replace('PYPI_VERSION', info['last_version'])
    meta = meta.replace('PYPI_URL', info['url'])
    meta = meta.replace('PYPI_SHA256', info['sha256'])
    with open(filepath, 'wt') as fout:
        fout.write(meta)
    logging.info(f'File {filepath} has been updated with informations from PyPi.')


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('-p', '--package', type=str, default='', required=True, help='The name of the package')
    parser.add_argument('-f', '--filename', type=str, default='.conda/meta.yaml', help='Path to meta.yaml, with filename')
    args = parser.parse_args()
    info = get_info(args.package)
    logging.info(f'Information of the last published PyPi package : {info}')
    replace_in_file(args.filename, info)



import os
from openfisca_tunisia import CountryTaxBenefitSystem as TunisiaTaxBenefitSystem

from openfisca_survey_manager.scenarios import AbstractSurveyScenario

survey_variables_filepath = os.path.join(
    os.path.dirname(os.path.abspath(__file__)),
    'data.py'
    )


class TunisiaSurveyScenario(AbstractSurveyScenario):
    id_variable_by_entity_key = dict(
        foyer_fiscal = 'id_foyer_fiscal',
        menage = 'id_menage',
        )
    role_variable_by_entity_key = dict(
        foyer_fiscal = 'role_foyer_fiscal',
        menage = 'role_menage',
        )

    weight_column_name_by_entity = dict(
        individu = 'poids',
        foyer_fiscal = 'poids_foyer_fiscal',
        menage = 'poids_menage',
        )

    def __init__(self, input_data_frame = None, tax_benefit_system = None,
            baseline_tax_benefit_system = None, year = None):
        super(TunisiaSurveyScenario, self).__init__()
        assert input_data_frame is not None
        assert year is not None
        self.year = year
        if tax_benefit_system is None:
            tax_benefit_system = TunisiaTaxBenefitSystem()

        tax_benefit_system.add_variables_from_file(survey_variables_filepath)
        if baseline_tax_benefit_system:
            baseline_tax_benefit_system.add_variables_from_file(survey_variables_filepath)

        self.set_tax_benefit_systems(
            tax_benefit_system = tax_benefit_system,
            baseline_tax_benefit_system = baseline_tax_benefit_system
            )

        self.used_as_input_variables = list(
            set(list(tax_benefit_system.variables.keys())).intersection(
                set(input_data_frame.columns)
                ))
        self.init_from_data_frame(input_data_frame = input_data_frame)
        self.new_simulation()
        if baseline_tax_benefit_system is not None:
            self.new_simulation(use_baseline = True)



from openfisca_core.entities import build_entity


Individu = build_entity(
    key = 'individu',
    plural = 'individus',
    label = 'Individu',
    is_person = True
    )

FoyerFiscal = build_entity(
    key = 'foyer_fiscal',
    plural = 'foyers_fiscaux',
    label = 'Foyer fiscal',
    roles = [
        {
            'key': 'declarant',
            'plural': 'declarants',
            'label': 'Déclarants',
            'subroles': ['declarant_principal', 'conjoint']
            },
        {
            'key': 'personne_a_charge',
            'plural': 'personnes_a_charge',
            'label': 'Personnes à charge'
            },
        ]
    )

Menage = build_entity(
    key = 'menage',
    plural = 'menages',
    label = 'Ménage',
    roles = [
        {
            'key': 'personne_de_reference',
            'label': 'Personne de référence',
            'max': 1
            },
        {
            'key': 'conjoint',
            'label': 'Conjoint',
            'max': 1
            },
        {
            'key': 'enfant',
            'plural': 'enfants',
            'label': 'Enfants'
            },
        {
            'key': 'ascendant',
            'plural': 'ascendants',
            'label': 'Ascendants'
            },
        {
            'key': 'autre',
            'plural': 'autres',
            'label': 'Autres'
            }
        ]
    )

entities = [Individu, FoyerFiscal, Menage]



from numpy import (
    maximum as max_,
    minimum as min_,
    round,
    )


from openfisca_tunisia.variables.base import * # noqa F401


class salaire_unique(Variable):
    value_type = bool
    entity = Menage
    label = 'Indicatrice de salaire unique'
    definition_period = YEAR

    def formula(menage, period):
        salaire_imposable_personne_de_reference = menage.personne_de_reference('salaire_imposable', period = period)
        salaire_imposable_conjoint = menage.conjoint('salaire_imposable', period = period)
        return xor_(salaire_imposable_personne_de_reference > 0, salaire_imposable_conjoint > 0)


# Allocations familiales

class prestations_familiales_enfant_a_charge(Variable):
    value_type = bool
    entity = Individu
    label = 'Enfant considéré à charge au sens des prestations familiales'
    definition_period = MONTH
    reference = 'http://www.cleiss.fr/docs/regimes/regime_tunisie_salaries.html'

    #    Jusqu'à l'âge de 16 ans sans conditions.
    #    Jusqu'à l'âge de 18 ans pour les enfants en apprentissage qui ne perçoivent pas une rémunération
    # supérieure à 75 % du SMIG.
    #    Jusqu'à l'âge de 21 ans pour les enfants qui fréquentent régulièrement un établissement secondaire,
    # supérieur, technique ou professionnel, à condition que les enfants n'occupent pas d'emplois salariés.
    #    Jusqu'à l'âge de 21 ans pour la jeune fille qui remplace sa mère auprès de ses frères et sœurs. TODO
    #    Sans limite d'âge et quelque soit leur rang pour les enfants atteints d'une infirmité ou d'une maladie
    # incurable et se trouvant, de ce fait, dans l'impossibilité permanente et absolue d'exercer un travail
    # lucratif, et pour les handicapés titulaires d'une carte d'handicapé qui ne sont pas pris en charge
    # intégralement par un organisme public ou privé benéficiant de l'aide de l'Etat ou des collectivités
    # locales.

    def formula(individu, period, parameters):
        # TODO à retravailler
        # age = individu('age', period)
        handicap = individu('handicap', period)
        est_enfant = individu.has_role(Menage.ENFANT)

        condition_enfant = (
            (age_individu <= 16)
            + (age_individu <= 18) * (salaire_individu <= .75 * smig_48h_mensuel)
            )
        condition_jeune_etudiant = (
            # (age_individu <= 21) * etudiant ou soeur au foyer
            )

        return (condition_enfant + condition_jeune_etudiant + handicap) * est_enfant


class af_nbenf(Variable):
    value_type = float
    entity = Menage
    label = "Nombre d'enfants au sens des allocations familiales"
    definition_period = YEAR

    def formula(menage, period, parameters):
        prestations_familiales_enfant_a_charge = menage.members(
            'prestations_familiales_enfant_a_charge', period)
        af_nbenf = max_(
            menage.sum(prestations_familiales_enfant_a_charge),
            3
            )
        return af_nbenf


class af(Variable):
    value_type = float
    entity = Menage
    label = 'Allocations familiales'
    definition_period = YEAR

    def formula(menage, period, parameters):
        af_nbenf = menage('af_nbenf', period = period)
        # Le montant trimestriel est calculé en pourcentage de la rémunération globale trimestrielle palfonnée
        # à 122 dinars
        # TODO: ajouter éligibilité des parents aux allocations familiales
        pf = parameters.prestations.contributives.prestations_familiales
        bm = min_(
            max_(
                menage.personne_de_reference('salaire_imposable', period),
                menage.conjoint('salaire_imposable', period),
                ) / 4,
            pf.af.plaf_trim
            )  # base trimestrielle
        # prestations familliales  # Règle d'arrondi ?
        af_1enf = round(bm * parameters.af.taux.enf1, 2)
        af_2enf = round(bm * parameters.af.taux.enf2, 2)
        af_3enf = round(bm * parameters.af.taux.enf3, 2)
        af_base = (
            (af_nbenf >= 1) * af_1enf
            + (af_nbenf >= 2) * af_2enf
            + (af_nbenf >= 3) * af_3enf
            )
        return 4 * af_base  # annualisé


class majoration_salaire_unique(Variable):
    value_type = float
    entity = Menage
    label = 'Majoration du salaire unique'
    definition_period = YEAR  # TODO trimestrialiser

    def formula(menage, period, parameters):
        salaire_unique = menage('salaire_unique', period = period)
        af_nbenf = menage('af_nbenf', period = period)
        P = parameters(period.start).prestations.contributives.prestations_familiales
        af_1enf = round(P.salaire_unique.enf1, 3)  # trimestrielle
        af_2enf = round(P.salaire_unique.enf2, 3)  # trimestrielle
        af_3enf = round(P.salaire_unique.enf3, 3)  # trimestrielle
        af = (
            (af_nbenf >= 1) * af_1enf
            + (af_nbenf >= 2) * af_2enf
            + (af_nbenf >= 3) * af_3enf
            )
        return 4 * af * salaire_unique  # annualisé


def _af_cong_naiss(age, _P):
    # _option={'age': ENFS}
    return 0


def _af_cong_jeun_trav(age, _P):
    #    Les salariés de moins de 18 ans du régime non agricole bénéficient de
    #    2 jours de congés par mois et au maximum 24 jours ouvrables,
    #    l'employeur se fera rembourser par la CNSS 12 jours de congés. Les
    #    salariés âgés de 18 à 20 ans bénéficient de 18 jours de congés
    #    ouvrables par an soit 6 jours remboursés à l'employeur par la CNSS.
    #    Le remboursement à l'employeur est effectué par la Caisse Nationale
    #    de Sécurité Sociale de l'avance faite en exécution de l'article 113
    #    alinéa 2 du Code du Travail.
    # , _option = {'age': ENFS}
    return 0


class contribution_frais_creche(Variable):
    value_type = float
    entity = Menage
    label = 'Contribution aux frais de crêche'
    definition_period = YEAR

    def formula(menage, period, parameters):
        month = period.last_month
        smig48 = parameters(period.start).marche_travail.smig_48h_mensuel  # TODO: smig 48H
        # TODO rework and test
        # Une prise en charge peut être accordée à la mère exerçant une
        # activité salariée et dont le salaire ne dépasse pas deux fois et demie
        # le SMIG pour 48 heures de travail par semaine. Cette contribution est
        # versée pour les enfants ouvrant droit aux prestations familiales et
        # dont l'âge est compris entre 2 et 36 mois. Elle s'élève à 15 dinars par
        # enfant et par mois pendant 11 mois.
        # , _option = {'age_en_mois': ENFS, 'sal': [CHEF, PART]}
        somme_salaire_imposable = (
            menage.personne_de_reference('salaire_imposable', period = month)
            + menage.conjoint('salaire_imposable', period = month)
            )
        age_en_mois = menage.members('age_en_mois', period = month)
        creche = parameters(period).prestations.contributives.prestations_familiales.creche
        age_en_mois_benjamin = menage.min(age_en_mois)[0]

        elig_age = (age_en_mois_benjamin <= creche.age_max) * (age_en_mois_benjamin >= creche.age_min)
        elig_sal = somme_salaire_imposable < creche.plaf * smig48
        return creche.montant * elig_age * elig_sal * min_(creche.duree, 12 - age_en_mois_benjamin)


class prestations_familiales(Variable):  # TODO add _af_cong_naiss, af_cong_jeun_trav
    value_type = float
    entity = Menage
    label = 'Prestations familales'
    definition_period = YEAR

    def formula(menage, period):
        af = menage('af', period = period)
        majoration_salaire_unique = menage('majoration_salaire_unique', period = period)
        contribution_frais_creche = menage('contribution_frais_creche', period = period)
        return af + majoration_salaire_unique + contribution_frais_creche


#
# Assurances sociales   Maladie
#

def prestationl(age, sal, parameters):
    '''
    Assurance sociale - prestation en espèces TODO: à compléter
    '''
    # , _option = {'age': ENFS}
    #    P = _P.as.maladie
    P = 0
    mal = 0
    smig = parameters.marche_travail.smig
    return mal * P.part * max(P.plaf_mult * smig, sal) * P.duree


def _as_maternite(age, sal, parameters):
    '''
    Assurance sociale - maternité  TODO: à compléter
    '''
    # P = parameters.as.maternite
    # smig = parameters.marche_travail.smig
    # return P.part*max(P.plaf_mult*smig,sal)*P.duree
    return 0


def _as_deces(sal, paramters):
    '''
    Assurance sociale - décès   # TODO: à compléter
    '''
    # P = _P.as.deces
    return 0



'''6. Revenus de valeurs mobilières et de capitaux mobiliers.'''


from openfisca_tunisia.variables.base import *  # noqa analysis:ignore


class tspr(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Traitements, salaires, indemnités, pensions et rentes viagères'
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        revenu_assimile_salaire_apres_abattements = foyer_fiscal(
            'revenu_assimile_salaire_apres_abattements', period = period)
        revenu_assimile_pension_apres_abattements = foyer_fiscal(
            'revenu_assimile_pension_apres_abattements', period = period)

        return revenu_assimile_salaire_apres_abattements + revenu_assimile_pension_apres_abattements


class revenu_assimile_salaire(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Revenu assimilé à des salaires'
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        salaire_imposable = foyer_fiscal.declarant_principal('salaire_imposable', period = period, options = [ADD])
        salaire_en_nature = foyer_fiscal.declarant_principal('salaire_en_nature', period = period, options = [ADD])
        return (salaire_imposable + salaire_en_nature)


class smig(Variable):
    value_type = bool
    entity = FoyerFiscal
    label = 'Indicatrice de SMIG ou SMAG déduite du montant des salaires'
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        revenu_assimile_salaire = foyer_fiscal('revenu_assimile_salaire', period = period)
        salarie_declarant_percevoir_smig = foyer_fiscal.declarant_principal('salarie_declarant_percevoir_smig', period = period.first_month)
        smig_40h_mensuel = parameters(period.start).marche_travail.smig_40h_mensuel
        smig = (
            salarie_declarant_percevoir_smig
            + (revenu_assimile_salaire <= 12 * smig_40h_mensuel)
            )
        return smig


class revenu_assimile_salaire_apres_abattements(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Revenu imposé comme des salaires net des abatements'
    definition_period = YEAR

    def formula_2011(foyer_fiscal, period, parameters):
        revenu_assimile_salaire = foyer_fiscal('revenu_assimile_salaire', period = period)
        smig = foyer_fiscal('smig', period = period)
        tspr = parameters(period.start).impot_revenu.tspr
        abattement_frais_professionnels = min_(revenu_assimile_salaire * tspr.abat_sal, tspr.max_abat_sal)
        revenu_assimile_salaire_apres_abattements = max_(
            (
                revenu_assimile_salaire
                - abattement_frais_professionnels
                - max_(
                    smig * tspr.abattement_pour_salaire_minimum,
                    (revenu_assimile_salaire <= tspr.smig_ext) * tspr.abattement_pour_salaire_minimum
                    )
                ),
            0
            )
        return revenu_assimile_salaire_apres_abattements

    def formula(foyer_fiscal, period, parameters):
        revenu_assimile_salaire = foyer_fiscal('revenu_assimile_salaire', period = period)
        smig = foyer_fiscal('smig', period = period)
        tspr = parameters(period.start).impot_revenu.tspr
        abattement_frais_professionnels = min_(revenu_assimile_salaire * tspr.abat_sal, tspr.max_abat_sal)
        revenu_assimile_salaire_apres_abattements = min_(
            revenu_assimile_salaire
            - abattement_frais_professionnels
            - smig * tspr.abattement_pour_salaire_minimum,
            0
            )
        return revenu_assimile_salaire_apres_abattements


class revenu_assimile_pension_apres_abattements(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Revenu assimilé à des pensions après abattements'
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        revenu_assimile_pension = foyer_fiscal.declarant_principal('revenu_assimile_pension', period = period)
        avantages_nature_assimile_pension = foyer_fiscal.declarant_principal(
            'avantages_nature_assimile_pension', period = period)
        tspr = parameters(period.start).impot_revenu.tspr
        return (revenu_assimile_pension + avantages_nature_assimile_pension) * (1 - tspr.abat_pen)



import os

from openfisca_core.taxbenefitsystems import TaxBenefitSystem
from openfisca_tunisia import entities


COUNTRY_DIR = os.path.dirname(os.path.abspath(__file__))


class TunisiaTaxBenefitSystem(TaxBenefitSystem):
    '''Tunisian tax benefit system'''
    CURRENCY = 'DT'  # code iso is TND see https://fr.wikipedia.org/wiki/Dinar_tunisien
    REFORMS_DIR = os.path.join(COUNTRY_DIR, 'reformes')

    def __init__(self):
        # We initialize our tax and benefit system with the general constructor
        super(TunisiaTaxBenefitSystem, self).__init__(entities.entities)

        # We add to our tax and benefit system all the variables
        self.add_variables_from_directory(os.path.join(COUNTRY_DIR, 'variables'))

        # We add to our tax and benefit system all the legislation parameters defined in the  parameters files
        param_path = os.path.join(COUNTRY_DIR, 'parameters')
        self.load_parameters(param_path)



from openfisca_tunisia.variables.base import *


# Autres revenus


class salaire_etranger(Variable):
    value_type = int
    label = "Salaires perçus à l'étranger"
    entity = Individu
    definition_period = YEAR


class pension_etranger_non_transferee(Variable):
    value_type = int
    label = "Pensions perçues à l'étranger (non transférées)"
    entity = Individu
    definition_period = YEAR


class pension_etranger_transferee(Variable):
    value_type = int
    label = "Pensions perçues à l'étranger (transférées en Tunisie)"
    entity = Individu
    definition_period = YEAR


class autres_revenus_etranger(Variable):
    value_type = int
    label = "Autres revenus perçus à l'étranger"
    entity = Individu
    definition_period = YEAR

# Revenus exonérés
# Revenus non imposables

# deficit antérieurs non déduits


class deficits_anterieurs_non_deduits(Variable):
    value_type = int
    label = 'Déficits des années antérieures non déduits'
    entity = Individu
    definition_period = YEAR



from openfisca_tunisia.variables.base import *  # noqa analysis:ignore
from openfisca_tunisia.variables.prelevements_obligatoires.impot_revenu.irpp import calcule_base_imposable


class contribution_sociale_solidarite(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Contribution sociale de solidarité'
    definition_period = YEAR

    def formula_2020_01_01(foyer_fiscal, period, parameters):
        impot_revenu_brut = bareme.calc(revenu_net_imposable)
        revenu_net_imposable = foyer_fiscal('revenu_net_imposable', period = period)
        bareme_irpp = parameters(period.start).impot_revenu.bareme.copy()
        bareme_css = parameters(period.start).prelevements_sociaux.contribution_sociale_solidarite.salarie
        bareme_irpp.add_tax_scale(bareme_css)
        non_exonere_css = (
            revenu_net_imposable
            > parameters(period.start).impot_revenu.exoneration.seuil
            )
        return non_exonere_css * (
            non_exonere_irpp * bareme_irpp.calc(revenu_net_imposable)
            - impot_revenu_brut
            )

    def formula_2018_01_01(foyer_fiscal, period, parameters):
        revenu_net_imposable = foyer_fiscal('revenu_net_imposable', period = period)
        bareme_irpp = parameters(period.start).impot_revenu.bareme.copy()
        impot_revenu_brut = bareme_irpp.calc(revenu_net_imposable)
        non_exonere_irpp = revenu_net_imposable >= 0
        bareme_css = parameters(period.start).prelevements_sociaux.contribution_sociale_solidarite.salarie
        bareme_irpp.add_tax_scale(bareme_css)
        contribution_sociale_solidarite = (
            non_exonere_irpp * bareme_irpp.calc(revenu_net_imposable)
            - impot_revenu_brut
            )
        return contribution_sociale_solidarite


class contribution_sociale_solidarite_nette_a_payer(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Contribution sociale de solidarité nette des prélèvements à la source'
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        return (
            foyer_fiscal('contribution_sociale_solidarite', period)
            - foyer_fiscal.sum(foyer_fiscal.members('contribution_sociale_solidarite_prelevee_a_la_source', period, options = [ADD]))
            )


class contribution_sociale_solidarite_prelevee_a_la_source(Variable):
    value_type = float
    entity = Individu
    label = 'Contribution sociale de solidarité'
    definition_period = MONTH

    def formula_2020_01_01(individu, period, parameters):
        salaire_imposable = individu('salaire_imposable', period = period)
        deduction_famille_annuelle = individu.foyer_fiscal('deduction_famille', period = period.this_year)
        irpp_salarie_preleve_a_la_source = individu('irpp_salarie_preleve_a_la_source', period = period)
        non_exonere_irpp, revenu_assimile_salaire_apres_abattement = calcule_base_imposable(
            salaire_imposable, deduction_famille_annuelle, period, parameters)
        bareme_irpp = parameters(period.start).impot_revenu.bareme.copy()
        bareme_css = parameters(period.start).prelevements_sociaux.contribution_sociale_solidarite.salarie
        bareme_irpp.add_tax_scale(bareme_css)
        non_exonere_css = (
            (12 * revenu_assimile_salaire_apres_abattement - deduction_famille_annuelle)
            > parameters(period.start).impot_revenu.exoneration.seuil
            )
        return non_exonere_css * (
            non_exonere_irpp * bareme_irpp.calc(
                (12 * revenu_assimile_salaire_apres_abattement - deduction_famille_annuelle)
                ) / 12
            - irpp_salarie_preleve_a_la_source
            )

    def formula_2018_01_01(individu, period, parameters):
        '''
        Art. 53
        Pour les personnes physiques, la différence entre l’impôt sur le revenu déterminé
        sur la base du barème de l’impôt sur le revenu prévu à l’article 44 du code
        de l’impôt sur le revenu des personnes physiques et de l’impôt sur les sociétés,
        en majorant de un point les taux d’imposition applicables aux tranches de revenu
        prévues par ledit barème et l’impôt sur le revenu déterminé sur la base
        dudit barème d’impôt sans la majoration d'un point des taux d’imposition,

        Art. 39 de la loi de finance 2020
        La contribution sociale de solidarité ne s'applique pas aux personnes physiques qui réalisent exclusivement
        les revenus prévus à l'article 25 du code de l'impôt sur le revenu des personnes physiques
        et de l'impôt sur les sociétés et dont le revenu annuel net ne dépasse pas 5000 dinars
        après déduction des abattements au titre de la situation et charges de famille
        prévus à l'article 40 dudit code uniquement.
        '''
        salaire_imposable = individu('salaire_imposable', period = period)
        deduction_famille_annuelle = individu.foyer_fiscal('deduction_famille', period = period.this_year)
        irpp_salarie_preleve_a_la_source = individu('irpp_salarie_preleve_a_la_source', period = period)

        non_exonere_irpp, revenu_assimile_salaire_apres_abattement = calcule_base_imposable(
            salaire_imposable, deduction_famille_annuelle, period, parameters)
        bareme_irpp = parameters(period.start).impot_revenu.bareme.copy()
        bareme_css = parameters(period.start).prelevements_sociaux.contribution_sociale_solidarite.salarie
        bareme_irpp.add_tax_scale(bareme_css)
        contribution_sociale_solidarite = (
            non_exonere_irpp * bareme_irpp.calc(
                (12 * revenu_assimile_salaire_apres_abattement - deduction_famille_annuelle)
                ) / 12
            - irpp_salarie_preleve_a_la_source
            )
        return contribution_sociale_solidarite



from openfisca_tunisia.variables.base import *  # noqa analysis:ignore


class impots_directs(Variable):
    value_type = float
    entity = Individu
    label = 'Impôts directs'
    definition_period = YEAR

    def formula(individu, period):
        irpp = individu.foyer_fiscal('irpp', period = period)

        return irpp


class prestations_sociales(Variable):
    value_type = int
    entity = Individu
    label = 'Prestations sociales'
    definition_period = YEAR


class revenu_disponible(Variable):
    value_type = float
    entity = Menage
    label = 'Revenu disponible du ménage'
    definition_period = YEAR

    def formula(menage, period):
        revenu_disponible_individuels = menage.members('revenu_disponible_individuel', period = period)
        return menage.sum(revenu_disponible_individuels)


class revenu_disponible_individuel(Variable):
    value_type = float
    entity = Individu
    label = 'Revenu disponible individuel'
    definition_period = YEAR

    def formula(individu, period):
        revenus_du_travail = individu('revenus_du_travail', period = period)
        revenu_assimile_pension = individu('revenu_assimile_pension', period = period)
        revenus_du_capital = individu('revenus_du_capital', period = period)
        prestations_sociales = individu('prestations_sociales', period = period)
        impots_directs = individu('impots_directs', period = period)

        return (
            revenus_du_travail
            + revenu_assimile_pension
            + revenus_du_capital
            + prestations_sociales
            + impots_directs
            )


class revenus_du_capital(Variable):
    value_type = float
    entity = Individu
    label = 'Revenus du capital'
    definition_period = YEAR

    def formula(individu, period):
        revenus_fonciers = individu.foyer_fiscal('revenus_fonciers', period = period)
        return revenus_fonciers


class revenus_du_travail(Variable):
    value_type = float
    entity = Individu
    label = 'Revenus du travail'
    definition_period = YEAR

    def formula(individu, period):
        salaire_imposable = individu('salaire_imposable', period = period, options = [ADD])
        contribution_sociale_solidarite = individu('contribution_sociale_solidarite_prelevee_a_la_source', period = period, options = [ADD])
        return salaire_imposable - contribution_sociale_solidarite  # + beap + bic + bnc  TODO



'''Revenus de valeurs mobilières et de capitaux mobiliers.'''


from openfisca_tunisia.variables.base import *


# A Revenus des valeurs mobilières et de capitaux mobiliers


class valm_nreg(Variable):
    value_type = int
    label = 'Revenus des valeurs mobilières autres que ceux régulièrement distribués'
    entity = Individu
    definition_period = YEAR


class valm_jpres(Variable):
    value_type = int
    label = 'Jetons de présence'
    entity = Individu
    definition_period = YEAR


class valm_aut(Variable):
    value_type = int
    label = 'Autres rémunérations assimilées'
    entity = Individu
    definition_period = YEAR


# B Revenus de capitaux mobiliers


class capm_banq(Variable):
    value_type = int
    label = "Intérêts bruts des comptes spéciaux d'épargne ouverts auprès des banques"
    entity = Individu
    definition_period = YEAR


class capm_cent(Variable):
    value_type = int
    label = "Intérêts bruts des comptes spéciaux d'épargne ouverts auprès de la CENT"
    entity = Individu
    definition_period = YEAR


class capm_caut(Variable):
    value_type = int
    label = 'Intérêts des créances et intérêts et rémunérations des cautionnements'
    entity = Individu
    definition_period = YEAR


class capm_part(Variable):
    value_type = int
    label = 'Intérêts des titres de participation'
    entity = Individu
    definition_period = YEAR


class capm_oblig(Variable):
    value_type = int
    label = 'Intérêts des emprunts obligataires'
    entity = Individu
    definition_period = YEAR


class capm_caisse(Variable):
    value_type = int
    label = 'Intérêts des bons de caisse'
    entity = Individu
    definition_period = YEAR


class capm_plfcc(Variable):
    value_type = int
    label = 'Revenus des parts et de liquidation du fonds commun des créances'
    entity = Individu
    definition_period = YEAR


class capm_epinv(Variable):
    value_type = int
    label = "Intérêts des comptes épargne pour l'investissement"
    entity = Individu
    definition_period = YEAR


class capm_aut(Variable):
    value_type = int
    label = 'Autres intérêts'
    entity = Individu
    definition_period = YEAR



'''1. Bénéfices industriels et commerciaux.'''


from openfisca_tunisia.variables.base import *  # noqa analysis:ignore


class bic(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Bénéfices industriels et commerciaux (BIC)'
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        bic_reel_res_i = foyer_fiscal.members('bic_reel_res', period = period)
        bic_reel_res = foyer_fiscal.sum(bic_reel_res_i)
        # TODO:
        #    return bic_reel + bic_simpl + bic_forf
        return bic_reel_res

# régime réel
# régime réel simplifié
# régime forfaitaire


class bic_ca_global(Variable):
    value_type = float
    entity = Individu
    label = 'Chiffre d’affaires global (BIC, cession de fond de commerce'
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        '''
        Chiffre d’affaires global
        des personnes soumises au régime forfaitaire ayant cédé le fond de commerce
        '''
        bic_ca_revente = foyer_fiscal('bic_ca_revente', period = period)
        bic_ca_autre = foyer_fiscal('bic_ca_autre', period = period)

        return bic_ca_revente + bic_ca_autre


class bic_res_cession(Variable):
    value_type = float
    entity = Individu
    label = 'Résultat (BIC, cession de fond de commerce)'
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        bic_ca_global = foyer_fiscal('bic_ca_global', period = period)
        bic_depenses = foyer_fiscal('bic_depenses', period = period)

        return max_(bic_ca_global - bic_depenses, 0)


class bic_benef_fiscal_cession(Variable):
    value_type = float
    entity = Individu
    label = 'Bénéfice fiscal (BIC, cession de fond de commerce)'
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        bic_res_cession = foyer_fiscal('bic_res_cession', period = period)
        bic_pv_cession = foyer_fiscal('bic_pv_cession', period = period)
        return bic_res_cession + bic_pv_cession


def _bic_res_net(bic_benef_fiscal_cession, bic_part_benef_sp):
    '''
    Résultat net BIC TODO: il manque le régime réel
    '''
    return bic_benef_fiscal_cession + bic_part_benef_sp



import logging

log = logging.getLogger(__name__)


def init_single_entity(scenario, axes = None, enfants = None, foyer_fiscal = None, menage = None, parent1 = None, parent2 = None, period = None):
    if enfants is None:
        enfants = []
    assert parent1 is not None

    foyers_fiscaux = {}
    menages = {}
    individus = {}

    count_so_far = 0
    for nth in range(0, 1):
        foyer_fiscal_nth = foyer_fiscal.copy() if foyer_fiscal is not None else {}
        menage_nth = menage.copy() if menage is not None else {}
        group = [parent1, parent2] + (enfants or [])
        for index, individu in enumerate(group):
            if individu is None:
                continue
            id = individu.get('id')
            if id is None:
                individu = individu.copy()
                id = 'ind{}'.format(index + count_so_far)
            individus[id] = individu
            if index <= 1:
                foyer_fiscal_nth.setdefault('declarants', []).append(id)
                if index == 0:
                    menage_nth['personne_de_reference'] = id
                else:
                    menage_nth['conjoint'] = id
            else:
                foyer_fiscal_nth.setdefault('personnes_a_charge', []).append(id)
                menage_nth.setdefault('enfants', []).append(id)

        count_so_far += len(group)
        foyers_fiscaux['ff{}'.format(nth)] = foyer_fiscal_nth
        menages['m{}'.format(nth)] = menage_nth

    dict = {
        'period': period,
        'foyers_fiscaux': foyers_fiscaux,
        'menages': menages,
        'individus': individus
        }
    if axes:
        dict['axes'] = axes
    scenario.init_from_dict(dict)
    return scenario



'''4. Revenus fonciers.'''


from openfisca_tunisia.variables.base import *  # noqa analysis:ignore


class revenus_fonciers(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Revenus fonciers'
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        foncier_reel_resultat_fiscal = foyer_fiscal.declarant_principal('foncier_reel_resultat_fiscal', period = period)
        foncier_forfait_bati_resultat_fiscal = foyer_fiscal('foncier_forfait_bati_resultat_fiscal', period = period)
        foncier_forfait_non_bati_resultat_fiscal = foyer_fiscal('foncier_forfait_non_bati_resultat_fiscal', period = period)
        foncier_societes_personnes = foyer_fiscal.declarant_principal('foncier_societes_personnes', period = period)

        return foncier_reel_resultat_fiscal + foncier_forfait_bati_resultat_fiscal + foncier_forfait_non_bati_resultat_fiscal + foncier_societes_personnes


class foncier_forfait_bati_resultat_fiscal(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Revenus fonciers net des immeubles bâtis'
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        foncier_forfaitaire_batis_recettes = foyer_fiscal.declarant_principal(
            'foncier_forfaitaire_batis_recettes', period = period)
        # foncier_forfaitaire_batis_reliquat = foyer_fiscal.declarant_principal(
        #     'foncier_forfaitaire_batis_reliquat', period = period)
        foncier_forfaitaire_batis_frais = foyer_fiscal.declarant_principal(
            'foncier_forfaitaire_batis_frais', period = period)
        foncier_forfaitaire_batis_taxe = foyer_fiscal.declarant_principal(
            'foncier_forfaitaire_batis_taxe', period = period)
        taux_deduction_frais = parameters(period.start).impot_revenu.foncier.bati.deduction_frais
        return max_(
            0,
            foncier_forfaitaire_batis_recettes * (1 - taux_deduction_frais)
            - foncier_forfaitaire_batis_frais
            - foncier_forfaitaire_batis_taxe
            )


class foncier_forfait_non_bati_resultat_fiscal(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Revenus fonciers net des terrains non bâtis'
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        foncier_forfaitaire_non_batis_recettes = foyer_fiscal.declarant_principal(
            'foncier_forfaitaire_non_batis_recettes', period = period)
        foncier_forfaitaire_non_batis_depenses = foyer_fiscal.declarant_principal(
            'foncier_forfaitaire_non_batis_depenses', period = period)
        foncier_forfaitaire_non_batis_taxe = foyer_fiscal.declarant_principal(
            'foncier_forfaitaire_non_batis_taxe', period = period)
        return max_(
            foncier_forfaitaire_non_batis_recettes - foncier_forfaitaire_non_batis_depenses - foncier_forfaitaire_non_batis_taxe,
            0
            )



import re
import tomli
# This script prints the minimal version of Openfisca-Core to ensure their compatibility during CI testing
with open('./pyproject.toml', 'rb') as file:
    config = tomli.load(file)
    deps = config['project']['dependencies']
    for dep in deps:
        version = re.search(r'openfisca-core\[([^\]]+)\]\s*>=\s*([\d\.]*)', dep)
        if version:
            try:
                print(f'openfisca-core[{version[1]}]=={version[2]}')  # noqa: T201 <- This is to avoid flake8 print detection.
            except Exception as e:
                print(f'Error processing "{dep}": {e}')  # noqa: T201 <- This is to avoid flake8 print detection.
                exit(1)



from openfisca_tunisia.variables.base import *


class code_postal(Variable):
    value_type = int
    entity = Menage
    label = 'Localisation du logement (code postal)'
    definition_period = ETERNITY


class loyer(Variable):
    value_type = int
    entity = Menage
    label = 'Loyer du logement'
    definition_period = YEAR


class statut_occupation_logement(Variable):
    value_type = int
    entity = Menage
    label = "Statut d'occupation du logement"
    definition_period = YEAR



from openfisca_tunisia.variables.base import *


class amen_social_presence_handicap_lourd(Variable):
    value_type = bool
    entity = Menage
    label = 'Ménage comprenant un membre avec un handicap lourd'
    definition_period = MONTH
    # Critères primaires du décret 2020-317 du 19 mai 2020
    # Handicap de niveau 3

    def formula_2020(menage, period, parameters):
        return menage.sum(menage.members('handicap', period) >= 3)


class amen_social_pas_d_achat_onereux(Variable):
    value_type = bool
    default_value = True
    entity = Menage
    label = "Ménage n'ayant pas fait d'achat onéreux"
    definition_period = ETERNITY
    # Critères primaires du décret 2020-317 du 19 mai 2020
    # Ni le chef du ménage ni aucun membre de son ménage n’a effectué une transaction d’achat
    # ou de vente d’un bien mobilier ou immobilier dont la valeur dépasse 30 fois le SMIG. Ce
    # critère est vérifiable actuellement uniquement pour les véhicules immatriculés par les
    # recoupements administratifs avec l’Agence Technique des Transports Terrestres (ATTT).


class amen_social_pas_de_residence_secondaire(Variable):
    value_type = bool
    default_value = True
    entity = Menage
    label = 'Ménage ne possédant pas de résidence secondaire'
    definition_period = ETERNITY
    # Critères primaires du décret 2020-317 du 19 mai 2020
    # Le ménage n’est pas propriétaire d’un logement secondaire


class amen_social_eligible(Variable):
    value_type = bool
    entity = Menage
    label = 'Ménage éligible au programme Amen social'
    definition_period = MONTH

    def formula_2020(menage, period, parameters):
        # Critères primaires du décret 2020-317 du 19 mai 2020
        pas_de_residence_secondaire = menage('amen_social_pas_de_residence_secondaire', period)
        pas_d_achat_onereux = menage('amen_social_pas_d_achat_onereux', period)

        # Citère du revenu selon présence ou non de handicap lourd
        presence_handicap_lourd = menage('amen_social_presence_handicap_lourd', period)
        taille_menage = menage.nb_persons()
        revenu_menage = menage('amen_social_revenu', period)
        seuil_de_revenu = parameters(period).prestations.non_contributives.amen_social.eligibilite
        smig_mensuel = parameters(period.start).marche_travail.smig_40h_mensuel
        conditions_sans_handicap = [
            taille_menage == 1,
            taille_menage == 2,
            (taille_menage == 3) + (taille_menage == 4),
            taille_menage >= 5,
            ]
        valeurs_choisies_sans_handicap = [
            smig_mensuel * seuil_de_revenu.un_membre,
            smig_mensuel * seuil_de_revenu.deux_membres,
            smig_mensuel * seuil_de_revenu.trois_quatre_membres,
            smig_mensuel * seuil_de_revenu.plus_de_cinq_membres,
            ]

        conditions_avec_handicap = [
            (taille_menage == 1) * presence_handicap_lourd,
            (taille_menage == 2) * presence_handicap_lourd,
            ((taille_menage == 3) + (taille_menage == 4)) * presence_handicap_lourd,
            (taille_menage >= 5) * presence_handicap_lourd,
            ]
        valeurs_choisies_avec_handicap = [
            smig_mensuel * seuil_de_revenu.handicap_lourd.un_membre,
            smig_mensuel * seuil_de_revenu.handicap_lourd.deux_membres,
            smig_mensuel * seuil_de_revenu.handicap_lourd.trois_quatre_membres,
            smig_mensuel * seuil_de_revenu.handicap_lourd.plus_de_cinq_membres,
            ]

        critere_revenu = where(
            presence_handicap_lourd,
            revenu_menage <= select(conditions_avec_handicap, valeurs_choisies_avec_handicap),
            revenu_menage <= select(conditions_sans_handicap, valeurs_choisies_sans_handicap)
            )
        return pas_d_achat_onereux * pas_de_residence_secondaire * critere_revenu


class amen_social_revenu(Variable):
    value_type = float
    entity = Menage
    label = 'Revenu du ménage au sens du programme Amen social'
    definition_period = MONTH

    def formula_2020(menage, period, parameters):
        revenu_menage = (
            menage.sum(menage.members('salaire_net_a_payer', period))
            + menage.sum(menage.members('pension_de_retraite', period))
            )
        return revenu_menage


class transfert_monetaire_permanent_eligible(Variable):
    value_type = bool
    entity = Menage
    label = 'Ménage éligible au transfert monétaire permanent du programme Amen social'
    definition_period = MONTH

    def formula_2020(menage, period):
        return (
            menage('transfert_monetaire_permanent_eligibilite_score', period)
            + menage('transfert_monetaire_permanent_eligibilite_supplementaire', period)
            )


class transfert_monetaire_permanent_eligibilite_score(Variable):
    value_type = bool
    entity = Menage
    label = 'Ménage éligible au programme Amen social selon le modèle de ciblage'
    definition_period = MONTH

    def formula_2022_06(menage, period, parameters):
        decile = parameters(period).prestations.non_contributives.amen_social.decile
        return (
            menage('amen_social_eligible', period)
            + (menage('amen_social_score_decile', period) <= decile)
            )


class transfert_monetaire_permanent_eligibilite_supplementaire(Variable):
    value_type = bool
    entity = Menage
    label = "Ménage éligible au programme Amen social selon les critères supplémentaires d'éligibilité"
    definition_period = MONTH


class amen_social_enfants_a_charge(Variable):
    value_type = int
    entity = Menage
    label = 'Enfant considéré à charge au sens de la prestation Amen social'
    definition_period = MONTH

    def formula_2020(menage, period, parameters):
        age = menage.members('age', period)
        # Les enfants sont considérés comme elève dès 6 ans  (pris quand ?)
        # eleve = menage.members('eleve', period.this_year)
        etudiant = menage.members('etudiant', period.this_year)
        handicap = menage.members('handicap', period.this_year) == 3
        amen_social = parameters(period).prestations.non_contributives.amen_social.supplements
        condition_enfant = (age >= 6) * (age <= amen_social.limite_age_enfant)  # * eleve
        condition_jeune_etudiant = (age <= amen_social.limite_age_etudiant) * etudiant
        enfant_a_charge = condition_enfant + condition_jeune_etudiant + handicap
        return menage.sum(1 * enfant_a_charge, role = Menage.ENFANT)


class amen_social_enfants_handicapes_a_charge(Variable):
    value_type = int
    entity = Menage
    label = 'Enfant considéré à charge au sens de la prestation Amen social'
    definition_period = MONTH

    def formula_2020(menage, period, parameters):
        age = menage.members('age', period)
        handicap = menage.members('handicap', period.this_year) == 3
        amen_social = parameters(period).prestations.non_contributives.amen_social.supplements
        condition_enfant = (age >= 6) * (age <= amen_social.limite_age_enfant)
        condition_jeune_etudiant = (age <= amen_social.limite_age_etudiant)
        enfant_a_charge_handicape = (condition_enfant + condition_jeune_etudiant) * handicap
        return menage.sum(enfant_a_charge_handicape, role = Menage.ENFANT)


class transfert_monetaire_permanent(Variable):
    value_type = float
    entity = Menage
    label = 'Transfert monétaire permanent mensuel du programme Amen social'
    definition_period = MONTH

    def formula_2020(menage, period, parameters):
        amen_social = parameters(period).prestations.non_contributives.amen_social
        supplements = amen_social.supplements
        eligible = menage('transfert_monetaire_permanent_eligible', period)
        enfants_a_charge = menage('amen_social_enfants_a_charge', period)
        enfants_handicapes_a_charge = menage('amen_social_enfants_handicapes_a_charge', period)
        tmp = eligible * (
            amen_social.allocation_base
            + enfants_a_charge * supplements.amen_social_enfants_a_charge
            + enfants_handicapes_a_charge * supplements.amen_social_enfants_a_charge * supplements.handicap
            )
        return tmp

# TODO

# - séparer éligibilité AMEN et TMP
# - Coder éligibilité AMG1 et AMG2

# Amen regroupe amg1 amg2 et tmp

# sante_amg1

# sante_amg2
# - Limiter  Personne à charge : que enfant à charge
#
# Enfant en âge de scolairite que par âge 6- 18 an sauf demande attestaion pour rentrée scolaire
# pour prestation monétaire ou en nature

# - Rajouter allocations familiales
# Allocaion familiales
# Critère d'éligibilité inscrit à Amen et IS et décret 3.17 valide.
# 30 DT - de 0 à 18.
# Pas de score


# - Niveau de handicap
# handicap_leger, handicap_lourd



from openfisca_tunisia.variables.base import * # noqa F401
from openfisca_tunisia.scenarios import init_single_entity
from tests import base

import datetime


def test_contribution_frais_creche():
    year = 2011
    month = '2011-01'
    simulation = init_single_entity(
        base.tax_benefit_system.new_scenario(),
        period = year,
        parent1 = dict(age = 40),
        parent2 = dict(
            date_naissance = datetime.date(1972, 1, 1),
            ),
        enfants = [
            dict(
                # id = 'Behija',
                date_naissance = '1992-01-01',
                ),
            dict(
                # id = 'Hassen',
                date_naissance = '2002-01-01',
                ),
            dict(
                # id = 'Thoura',
                date_naissance = '2008-12-31',
                )
            ]
        ).new_simulation(debug = True)

    ages_en_mois = simulation.menage.members('age_en_mois', period = month)
    age_benjamin = simulation.menage.min(ages_en_mois)[0]
    assert age_benjamin == 24
    assert age_benjamin <= base.tax_benefit_system.parameters(year).prestations.contributives.prestations_familiales.creche.age_max

    contribution_frais_creche = simulation.calculate('contribution_frais_creche', period = year)
    assert contribution_frais_creche != 0


if __name__ == '__main__':
    test_contribution_frais_creche()



from openfisca_tunisia.variables.base import (
    Individu,
    MONTH,
    Reform,
    set_input_divide_by_period,
    Variable,
    )

try:
    from scipy.optimize import fsolve
except ImportError:
    fsolve = None


def calculate_net_from(salaire_de_base, individu, period, requested_variable_names):
    # Work in isolation
    temp_simulation = individu.simulation.clone()
    temp_individu = temp_simulation.individu

    # Calculated variable holders might contain undesired cache
    # (their entity.simulation points to the original simulation above)
    for name in requested_variable_names:
        temp_individu.get_holder(name).delete_arrays(period)

    # Clean 'computation_stack', that is used by -core to check for computation cycles.
    # The variable 'salaire_imposable' might have been called from variable X,
    # that will be calculated again in our iterations to compute the 'salaire_net_a_payer'
    # requested as an input variable, hence producing a cycle error
    temp_simulation.computation_stack = []

    temp_individu.get_holder('salaire_de_base').set_input(period, salaire_de_base)

    # Force recomputing of salaire_net_a_payer
    temp_individu.get_holder('salaire_net_a_payer').delete_arrays()
    net = temp_individu('salaire_net_a_payer', period)[0]

    return net


class salaire_de_base(Variable):
    value_type = float
    entity = Individu
    label = 'Salaire de base'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period):
        # Use numerical inversion to calculate 'salaire_de_base' from 'salaire_net_a_payer'
        net = individu.get_holder('salaire_net_a_payer').get_array(period)

        if net is None:
            return individu.empty_array()

        simulation = individu.simulation
        simulation.period = period

        # List of variables already calculated.
        # We will need it to remove their holders, that might contain undesired cache
        requested_variable_names = [stack_frame['name'] for stack_frame in simulation.tracer.stack]

        def solve_func(net):
            def innerfunc(essai_salaire_de_base):
                return calculate_net_from(essai_salaire_de_base, individu, period, requested_variable_names) - net
            return innerfunc

        salaire_de_base_calcule = \
            fsolve(
                solve_func(net),
                net * 1,  # first guess
                xtol = 1 / 1000,  # précision au millime
                )

        return salaire_de_base_calcule


class de_net_a_salaire_de_base(Reform):
    name = 'Inversion du calcul brut (salaire_de_base) -> net'

    def apply(self):
        self.update_variable(salaire_de_base)



from openfisca_tunisia.variables.base import *


# Pensions


class pension_de_retraite(Variable):
    value_type = float
    label = 'Pension'
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period


class pension_orphelin(Variable):
    value_type = float
    label = 'Pension'
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period


class pension_d_invalidite(Variable):
    value_type = float
    label = 'Pension'
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period


class pension(Variable):
    value_type = float
    label = 'Pension'
    entity = Individu
    definition_period = YEAR
    set_input = set_input_divide_by_period

    def formula(individu, period):
        return (
            individu('pension_de_retraite', period, options = [ADD])
            + individu('pension_d_invalidite', period, options = [ADD])
            + individu('pension_orphelin', period, options = [ADD])
            )


class revenu_assimile_pension(Variable):
    value_type = float
    label = 'Revenus assimilés à des pensions (pensions et rentes viagères)'
    entity = Individu
    definition_period = YEAR

    def formula_2025_01(individu, period):
        return individu('pension_de_retraite', period, options = [ADD])

    def formula(individu, period):
        return individu('pension', period, options = [ADD])


class avantages_nature_assimile_pension(Variable):
    value_type = float
    label = 'Avantages en nature assimilables à des pensions'
    entity = Individu
    definition_period = YEAR



# Read package version in pyproject.toml and replace it in .conda/recipe.yaml

import argparse
import logging
import re

logging.basicConfig(level=logging.INFO, format='%(message)s')
PACKAGE_VERSION = 'X.X.X'
CORE_VERSION = '>=43,<44'
NUMPY_VERSION = '>=1.24.3,<2'


def get_versions():
    '''
    Read package version and deps in pyproject.toml
    '''
    openfisca_core_api = None
    openfisca_tunisia = None
    with open('./pyproject.toml', 'r') as file:
        content = file.read()
    # Extract the version of openfisca_tunisia
    version_match = re.search(r'^version\s*=\s*"([\d.]*)"', content, re.MULTILINE)
    if version_match:
        openfisca_tunisia = version_match.group(1)
    else:
        raise Exception('Package version not found in pyproject.toml')
    # Extract dependencies
    version = re.search(r'openfisca-core\[web-api\]\s*((?:==|>=)\s*[\d\.]+(?:\s*,\s*<\s*\d+)?)"', content, re.MULTILINE)
    if version:
        openfisca_core_api = version.group(1)
    version = re.search(r'numpy\s*(>=\s*[\d\.]*,\s*<\d*)"', content, re.MULTILINE)
    if version:
        numpy = version.group(1)
    if not openfisca_core_api or not numpy:
        raise Exception('Dependencies not found in pyproject.toml')
    return {
        'openfisca_tunisia': openfisca_tunisia,
        'openfisca_core_api': openfisca_core_api.replace(' ', ''),
        'numpy': numpy.replace(' ', ''),
        }


def replace_in_file(filepath: str, info: dict):
    '''
    ::filepath:: Path to meta.yaml, with filename
    ::info:: Dict with information to populate
    '''
    with open(filepath, 'rt') as fin:
        meta = fin.read()
    # Replace with info from pyproject.toml
    if PACKAGE_VERSION not in meta:
        raise Exception(f'{PACKAGE_VERSION=} not found in {filepath}')
    meta = meta.replace(PACKAGE_VERSION, info['openfisca_tunisia'])
    if CORE_VERSION not in meta:
        raise Exception(f'{CORE_VERSION=} not found in {filepath}')
    meta = meta.replace(CORE_VERSION, info['openfisca_core_api'])
    if NUMPY_VERSION not in meta:
        raise Exception(f'{NUMPY_VERSION=} not found in {filepath}')
    meta = meta.replace(NUMPY_VERSION, info['numpy'])
    with open(filepath, 'wt') as fout:
        fout.write(meta)
    logging.info(f'File {filepath} has been updated with informations from pyproject.toml.')


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('-r', '--replace', type=bool, default=False, required=False, help='replace in file')
    parser.add_argument('-f', '--filename', type=str, default='.conda/recipe.yaml', help='Path to recipe.yaml, with filename')
    parser.add_argument('-o', '--only_package_version', type=bool, default=False, help='Only display current package version')
    args = parser.parse_args()
    info = get_versions()
    file = args.filename
    if args.only_package_version:
        print(f'{info["openfisca_tunisia"]}')  # noqa: T201
        exit()
    logging.info('Versions :')
    print(info)  # noqa: T201
    if args.replace:
        logging.info(f'Replace in {file}')
        replace_in_file(file, info)
    else:
        logging.info('Dry mode, no replace made')



'''6. Revenus de valeurs mobilières et de capitaux mobiliers'''


from openfisca_tunisia.variables.base import *  # noqa analysis:ignore


class rvcm(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Revenus de valeurs mobilières et de capitaux mobiliers'
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        capm_banq = foyer_fiscal.declarant_principal('capm_banq', period = period)
        capm_cent = foyer_fiscal.declarant_principal('capm_cent', period = period)
        capm_caut = foyer_fiscal.declarant_principal('capm_caut', period = period)
        capm_part = foyer_fiscal.declarant_principal('capm_part', period = period)
        capm_oblig = foyer_fiscal.declarant_principal('capm_oblig', period = period)
        capm_caisse = foyer_fiscal.declarant_principal('capm_caisse', period = period)
        capm_plfcc = foyer_fiscal.declarant_principal('capm_plfcc', period = period)
        capm_epinv = foyer_fiscal.declarant_principal('capm_epinv', period = period)
        capm_aut = foyer_fiscal.declarant_principal('capm_aut', period = period)

        return (
            capm_banq
            + capm_cent
            + capm_caut
            + capm_part
            + capm_oblig
            + capm_caisse
            + capm_plfcc
            + capm_epinv
            + capm_aut
            )



import pytest
import datetime

from openfisca_tunisia.scenarios import init_single_entity
from tests.base import tax_benefit_system


scenarios_arguments = [
    dict(
        period = year,
        parent1 = dict(
            date_naissance = datetime.date(1972, 1, 1),
            salaire_de_base = 2000,
            regime_securite_sociale_cotisant = 'rsna',
            ),
        parent2 = dict(
            date_naissance = datetime.date(1972, 1, 1),
            ),
        )
    for year in range(datetime.date.today().year + 1, 2007, -1)
    ]


def check_run(simulation, period):
    assert simulation.calculate('revenu_disponible', period = period) is not None, \
        "Can't compute revenu_disponible on period {}".format(period)
    assert simulation.calculate_add('salaire_super_brut', period = period) is not None, \
        "Can't compute salaire_super_brut on period {}".format(period)


@pytest.mark.parametrize('one_scenario_arguments', scenarios_arguments)
def test_basics(one_scenario_arguments):
    scenario = init_single_entity(
        tax_benefit_system.new_scenario(),
        **one_scenario_arguments
        )
    simulation = scenario.new_simulation(debug = False)
    period = one_scenario_arguments['period']
    check_run(simulation, period)


@pytest.mark.parametrize('period', range(2007, datetime.date.today().year + 1))
def test_1_parent_axe(period):
    scenario = init_single_entity(
        tax_benefit_system.new_scenario(),
        axes = [[
            dict(
                count = 3,
                name = 'salaire_imposable',
                max = 100000,
                min = 0,
                )
            ]],
        period = period,
        parent1 = dict(date_naissance = datetime.date(period - 40, 1, 1)),
        )
    simulation = scenario.new_simulation()
    assert simulation.calculate('revenu_disponible', period = period) is not None, \
        "Can't compute revenu_disponible on period {}".format(period)
    assert simulation.calculate_add('salaire_super_brut', period = period) is not None, \
        "Can't compute salaire_super_brut on period {}".format(period)



import sys
from glob import glob


def split_tests(number_of_files, CI_NODE_TOTAL, CI_NODE_INDEX, test_files_list):
    test_files_sublist = []

    for file_index in range(number_of_files):
        file_number = file_index % CI_NODE_TOTAL
        if file_number == CI_NODE_INDEX:
            test_files_sublist.append(test_files_list[file_index])

    tests_to_run_string = ' '.join(test_files_sublist)

    return tests_to_run_string


if __name__ == '__main__':
    CI_NODE_TOTAL, CI_NODE_INDEX = int(sys.argv[1]), int(sys.argv[2])
    test_files_list = glob('tests/**/*.yaml', recursive=True) + glob('tests/**/*.yml', recursive=True)
    number_of_files = len(test_files_list)
    sys.stdout.write(split_tests(number_of_files, CI_NODE_TOTAL, CI_NODE_INDEX, test_files_list))



'''Aide médicale gratuite.'''


from openfisca_tunisia.variables.base import *


class amg_1(Variable):
    value_type = bool
    entity = Menage
    label = 'Aide médicale gratuite pour les ménages pauvres - AMG1'
    definition_period = MONTH


class amg_2(Variable):
    value_type = bool
    entity = Menage
    label = 'Programme d’assistance médicale à tarifs réduits les ménages vulnérables - AMG2'
    definition_period = MONTH



from openfisca_tunisia.variables.base import *


# BIC Bénéfices industriels et commerciaux
# régime réel


class TypesActivite(Enum):
    neant = 'Néant'
    commercant = 'Commerçant'
    industriel = 'Industriel'
    prestataire_de_services = 'Prestataire de services'
    artisan = 'Artisan'
    multi_activites = "Plus d'une activité"


class TypesCadreLegalActiviteEntreprise(Enum):
    exportation_totale = "Exportation totale dans le cadre du Code d'Incitations aux Investissements (CII)"
    developpement_regional = 'Développement régional'
    developpement_agricole = 'Développement agricole'
    parcs_activites_economiques = 'Parcs des activités économiques'
    exportation_droit_commun = 'Exportation dans le cadre du droit commun'
    autres = 'Autres (à préciser)'


class TypesStructureDeclarationInvestissement(Enum):
    api = "Agence de Promotion de l'Industrie et de l'Innovation"
    apia = 'Agence de Promotion des Investissements Agricoles'
    commissariat_regional_developpement_agricole = 'Commissariat régional du développement agricole'
    ont = 'ONT'
    autres = 'Autre structure (à préciser)'


class bic_reel(Variable):
    value_type = Enum
    possible_values = TypesActivite
    default_value = TypesActivite.neant
    entity = Individu
    label = "Type d'activité (BIC)"
    definition_period = YEAR


# Les personnes soumises au régime forfaitaire qui ont cédé le fond de commerce peuvent déclarer l’impôt
# annuel sur le revenu au titre des bénéfices industriels et commerciaux
# sur la base de la différence entre les recettes et les dépenses .
# régime des sociétés de personnes


class bic_societes_personnes(Variable):
    value_type = bool
    entity = Individu
    label = 'Indicatrice des sociétés de personnes et assimilées (BIC)'
    definition_period = YEAR


class cadre_legal(Variable):
    value_type = Enum
    default_value = TypesCadreLegalActiviteEntreprise.developpement_regional  # Développement régional
    possible_values = TypesCadreLegalActiviteEntreprise
    entity = Individu
    label = "Cadre légal de l'activité de l'entreprise"
    definition_period = YEAR


class bic_reel_res(Variable):
    value_type = int
    entity = Individu
    label = 'Résultat comptable (BIC, régime réel)'
    definition_period = YEAR


class bic_forfaitaire_resultat(Variable):
    value_type = int
    entity = Individu
    label = 'Résultat (BIC, régime forfaitaire, cession de fonds de commerce)'
    definition_period = YEAR


class bic_societes_personnes_resultat(Variable):
    value_type = int
    entity = Individu
    label = 'Part dans le bénéfice ou dans la perte des sociétés de personnes et assimilées ' \
        'exerçant dans le secteur industriel et commercial (BIC)'
    definition_period = YEAR


class structure_declaration_investissement(Variable):
    value_type = Enum
    possible_values = TypesStructureDeclarationInvestissement
    default_value = TypesStructureDeclarationInvestissement.api
    entity = Individu
    label = "Structure auprès de laquelle la déclaration d'investissement a été déposée"
    definition_period = YEAR


# A/ Régime réel
# Valeur du stock au début de l’exercice
# Valeur du stock à la fin de l’exercice
# Valeur des achats au cours de l’exercice
# Chiffre d’affaires local HT.
# Chiffre d’affaires à l’exportation
# Chiffre d’affaires global TTC.
# Chiffre d’affaires provenant des activités de services.
# Montant des primes (5) Primes octroyées dans le cadre du CII ou dans le cadre d'encouragement de l'Exportation
# ou dans le cadre d'un programme de mise à niveau approuvé ou dans le cadre des interventions
# du fonds national de l’emploi.
# Résultat comptable
# Résultat fiscal (6) Joindre à la déclaration l’état de détermination du résultat fiscal.


class bic_res_fiscal(Variable):
    value_type = int
    entity = Individu
    label = 'Résultat fiscal (BIC, régime réel)'
    definition_period = YEAR


# Case réserve aux personnes soumises au régime forfaitaire ayant cédé le fond de commerce


class bic_ca_revente(Variable):
    value_type = int
    entity = Individu
    label = "Chiffre d'affaires global au titre des activités d'achat en vue de la revente et les activités de transformation (BIC, régime forfaitaire, cession de fonds de commerce)"
    definition_period = YEAR


class bic_ca_autre(Variable):
    value_type = int
    entity = Individu
    label = 'Chiffre d’affaires global au titre d’autres activités (BIC, régime forfaitaire, cession de fonds de commerce)'
    definition_period = YEAR


class bic_depenses(Variable):
    value_type = int
    label = 'Total des dépenses (BIC, cession de fonds de commerce)'
    entity = Individu
    definition_period = YEAR


class bic_pv_cession(Variable):
    value_type = int
    entity = Individu
    label = 'Plue-value de cession du fond de commerce (BIC, régime forfaitaire, cession de fonds de commerce)'
    definition_period = YEAR


# B/ Part dans le bénéfice ou dans la perte des sociétés de personnes
# et assimilées exerçant dans le secteur industriel et commercial


class bic_part_benef_sp(Variable):
    value_type = int
    label = 'Part dans le bénéfice ou dans la perte des sociétés de personnes et assimilées exerçant ' \
        'dans le secteur industriel et commercial'
    entity = Individu
    definition_period = YEAR


# BNC Bénéfices des professions non commerciales
# A/ Régime réel
# - Chiffre d’affaires local HT.
# - Chiffre d’affaires à l’exportation
# - Chiffre d’affaires global TTC.
# - Montant des primes (1) Primes octroyées dans le cadre du CII ou dans le cadre d'encouragement de l'exportation
# ou dans le cadre d'un programme de mise à niveau approuvé ou dans le cadre des interventions
# du fonds national de l’emploi
# - Résultat comptable
# - Résultat fiscal (2) Joindre à la déclaration l'état de détermination du résultat fiscal


class bnc_reel_res_fiscal(Variable):
    value_type = int
    entity = Individu
    label = 'Résultat fiscal (BNC, régime réel)'
    definition_period = YEAR


# B/ Détermination du bénéfice sur la base d’une assiette forfaitaire
# - Recettes au titre des services locaux
# - Recettes au titre des services exportés (3) Pour les entreprises totalement exportatrices
# dans le cadre du CII ou exerçant dans les parcs d’activités économiques.
# - Recettes globales brutes TTC


class bnc_forfaitaire_recettes_brutes(Variable):
    value_type = int
    label = 'Recettes globales brutes TTC (BNC, assiette forfaitaire)'
    entity = Individu
    definition_period = YEAR


# - Montant des primes (1) Primes octroyées dans le cadre du CII ou dans le cadre d'encouragement de l'exportation
# ou dans le cadre d'un programme de mise à niveau approuvé ou dans le cadre des interventions
# du fonds national de l’emploi

# C/ Part dans le bénéfice ou dans la perte des sociétés de personnes et assimilées qui réalisent
# des bénéfices non commerciaux


class bnc_part_benef_sp(Variable):
    value_type = int
    entity = Individu
    label = 'Part dans le bénéfice ou dans la perte des sociétés de personnes et assimilées' \
        'qui réalisent des bénéfices non commerciaux (BNC)'
    definition_period = YEAR


# beap Bénéfices de l'exploitation agricole et de pêche
# A/ Régime réel
# - Chiffre d’affaires local
# - Chiffre d’affaires à l’exportation
# - Chiffre d’affaires global
# - Montant des primes  Primes octroyées dans le cadre du CII ou dans le cadre d'encouragement de l'exportation
# ou dans le cadre d'un programme de mise à niveau approuvé ou dans le cadre des interventions
# du fonds national de l’emploi.
# - Résultat comptable B = bénéfice P = perte
# - Résultat fiscal  B = bénéfice P = perte


class beap_reel_res_fiscal(Variable):
    value_type = int
    label = 'Résultat fiscal (BEAP, régime réel)'
    entity = Individu
    definition_period = YEAR

# B/ Détermination du bénéfice sur la base du reliquat positif entre les
# recettes et les dépenses
# - Recettes brutes …………………………..
# - Stocks …………………………..


class beap_reliq_rec(Variable):
    value_type = int
    label = 'Recettes brutes (BEAP, bénéfice comme reliquat positif entre recette et dépenses)'
    entity = Individu
    definition_period = YEAR


class beap_reliq_stock(Variable):
    value_type = int
    label = 'Stocks (BEAP, bénéfice comme reliquat positif entre recette et dépenses)'
    entity = Individu
    definition_period = YEAR

# TOTAL …………………………..
# - Déduction des dépenses d’exploitation justifiées …………………………..


class beap_reliq_dep_ex(Variable):
    value_type = int
    label = "Dépenses d'exploitation (BEAP, bénéfice comme reliquat positif entre recette et dépenses)"
    entity = Individu
    definition_period = YEAR

# - Montant des primes (1) …………………………..
# - Résultat B = bénéfice P = perte …………………………..
# - Bénéfice fiscal (4) …………………………..


class beap_reliq_benef_fiscal(Variable):
    value_type = int
    label = 'Bénéfice fiscal (BEAP, bénéfice comme reliquat positif entre recette et dépenses)'
    entity = Individu
    definition_period = YEAR

# C/ Détermination du bénéfice sur la base de monographies sectorielles (5)
# - Bénéfice fiscal …………………………..


class beap_monogr(Variable):
    value_type = int
    label = 'Bénéfice sur la base de monographies sectorielles (BEAP)'
    entity = Individu
    definition_period = YEAR
# D/ Part dans le bénéfice ou dans la perte des sociétés de personnes et
# assimilées exerçant dans le secteur agricole et de pêche


class beap_part_benef_sp(Variable):
    value_type = int
    label = 'Part dans le bénéfice ou dans la perte des sociétés de personnes et assimilées ' \
        'exerçant dans le secteur agricole et de pêche (BEAP)'
    entity = Individu
    definition_period = YEAR



from openfisca_tunisia.variables.base import *


# class TypesHandicap(Enum):
#     __order__ = 'neant leger intermediaire lourd'
#     # Needed to preserve the enum order in Python 2
#     neant = 'Néant'
#     leger = 'Léger'
#     intermediaire = 'Intermédiaire'
#     lourd = 'Lourd'


# class handicap(Variable):
#     value_type = Enum
#     possible_values = TypesHandicap
#     default_value = TypesHandicap.neant
#     label = 'Niveau de handicap'
#     entity = Individu
#     definition_period = ETERNITY


class handicap(Variable):
    value_type = int
    default_value = 0
    label = 'Niveau de handicap (0-Néant, 1-Léger, 2-intermédiraire, 3-Lourd)'
    entity = Individu
    definition_period = ETERNITY
